import urllib
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from datawarehouse_connector.constants import *
from datawarehouse_connector.utils import logger

class SnowflakeSession:
    def __init__(self, data):
        self.data = data

    def create_session(self):
        try:
            username = self.data.get(USERNAME)
            host = self.data.get(HOST)
            password = self.data.get(PASSWORD)
            role = self.data.get(ROLE)
            warehouse = self.data.get(WAREHOUSE)
            database = self.data.get(DATABASE)
            encoded_password = urllib.parse.quote(password, safe="")
            db_connection_string = (
                f"snowflake://{username}:{encoded_password}@{host}/{database}"
                f"?warehouse={warehouse}&role={role}"
            )
            engine = create_engine(db_connection_string)
            Session = sessionmaker(bind=engine)
            session = Session()
            if session:
                result = session.execute("SELECT CURRENT_VERSION()").fetchone()
                logger.info(f"Snowflake version: {result[0]}")
                return session
        except Exception as e:
            logger.exception(RED + " Failed to create Snowflake session " + ENDC)
            return None



DB_SOURCE = "db_source"
SNOWFLAKE = "snowflake"
REDSHIFT = "redshift"
POSTGRESQL = "postgresql"


HOST = "host"
USERNAME = "user"
PASSWORD = "password"
ROLE = "role"
WAREHOUSE = "warehouse"
DATABASE = "database"

PSYCOPG2_KEY = "psycopg2"
REDSHIFT_DRIVER_KEY = "redshift+redshift_connector"

SSH_HOST = 'ssh_host'
SSH_USERNAME = 'ssh_username'
SSH_PEM_FILE_CONTENT = 'ssh_pem_file_content'

MISSING_MSG = 'is missing from Required credentials for SSH tunnel connection'
LOCALHOST = "localhost"
#logging color scheme
GREEN = "\033[92m"
RED = '\033[91m'
YELLOW = "\033[93m"
ENDC = '\033[0m'


import os
import urllib
import logging
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.engine.url import URL
from datawarehouse_connector.constants import *
from datawarehouse_connector.connector import DatabaseSessionFactory

# logging
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


import os
import logging

# Configure logging
logging.basicConfig(level=logging.INFO)
# Suppress SQLAlchemy debug messages
logging.getLogger('sqlalchemy.engine').setLevel(logging.WARNING)
logger = logging.getLogger(__name__)

# Set environment variables
os.environ['SQLALCHEMY_SILENCE_UBER_WARNING'] = '1'



# setup.py
from setuptools import setup, find_packages

setup(
    name='datawarehouse_connector',
    version='0.1.3',
    packages=find_packages(),
    install_requires=[
        'psycopg2-binary',
        'redshift-connector',
        'snowflake-connector-python',
        'snowflake-sqlalchemy',
        'SQLAlchemy',
        'sqlalchemy-redshift',
        'urllib3'
    ],
    author='Chandani Kumari',
    author_email='Chandani@coditation.com',
    description='A package for database session management',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/Chandani7250/datawarehouse_connector',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',
)



import paramiko
from io import StringIO
import time
import threading
from sshtunnel import SSHTunnelForwarder
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.engine.url import URL
from datawarehouse_connector.constants import *
from datawarehouse_connector.utils import logger

keep_tunnel_event = True
class RedshiftSession:
    def __init__(self, data):
        self.data = data
        self.ssh_tunnel = None

    def create_ssh_tunnel(self):
        try:
            ssh_host = self.data.get(SSH_HOST)
            ssh_username = self.data.get(SSH_USERNAME)
            ssh_pem_file_content = self.data.get(SSH_PEM_FILE_CONTENT)
            redshift_host = self.data.get(HOST)
            redshift_port = 5439  # Redshift default port
            ssh_private_key = (
                paramiko.RSAKey(file_obj=StringIO(ssh_pem_file_content))
                if ssh_pem_file_content and ssh_pem_file_content != ""
                else None
            )
            with SSHTunnelForwarder(
                (ssh_host, 22),
                ssh_username=ssh_username,
                ssh_pkey=ssh_private_key,
                remote_bind_address=(redshift_host, int(redshift_port)),
                local_bind_address=(LOCALHOST, 5439),  # Use the same local port
            ) as ssh_tunnel:
                logger.info(YELLOW + " SSH tunnel established" + ENDC)
                global keep_tunnel_event
                while keep_tunnel_event:
                    time.sleep(1)
                else:
                    ssh_tunnel.stop()
                    print("stopped ssh_tunnel")
            logger.info("SSH tunnel established")
        except Exception as e:
            logger.exception(RED + " Failed to create SSH tunnel " + ENDC)
            raise

    def start_ssh_tunnel(self):
        ssh_thread = threading.Thread(target=self.create_ssh_tunnel)
        ssh_thread.daemon = True
        ssh_thread.start()
        
    def create_session(self):
        try:
            host = self.data.get(HOST)
            ssh_tunnel_connection = self.data.get('ssh_tunnel_connection')
            if ssh_tunnel_connection:
                if SSH_HOST in self.data and SSH_USERNAME in self.data and SSH_PEM_FILE_CONTENT in self.data:
                    self.start_ssh_tunnel()
                    time.sleep(5)
                    host = LOCALHOST
                else:
                    if SSH_HOST not in self.data:
                        print(SSH_HOST +' '+ MISSING_MSG)
                    elif SSH_USERNAME not in self.data:
                        print(SSH_USERNAME +' '+ MISSING_MSG)
                    elif SSH_PEM_FILE_CONTENT not in self.data:
                        print(SSH_PEM_FILE_CONTENT +' '+ MISSING_MSG)

            username = self.data.get(USERNAME)
            password = self.data.get(PASSWORD)
            database = self.data.get(DATABASE)
            db_connection_string = URL.create(
                drivername=REDSHIFT_DRIVER_KEY,
                host=host,
                port=5439,
                database=database,
                username=username,
                password=password,
                query={'sslmode': 'verify-ca'}  # Adjust sslmode as needed
            )
            engine = create_engine(db_connection_string)
            Session = sessionmaker(bind=engine)
            session = Session()
            if session:
                result = session.execute("SELECT version()").fetchone()
                logger.info(f"Redshift version: {result[0]}")
                return session
        except Exception as e:
            logger.exception(RED + " Failed to create Redshift session " + ENDC)
            return None
        finally:
            if self.ssh_tunnel:
                self.ssh_tunnel.stop()
                logger.info("SSH tunnel closed")

    def stop_ssh_tunnel(self):
        if self.ssh_tunnel:
            self.ssh_tunnel.stop()
            logger.info("SSH tunnel closed")


from datawarehouse_connector.constants import *
from datawarehouse_connector.snowflake_session import SnowflakeSession
from datawarehouse_connector.redshift_session import RedshiftSession
from datawarehouse_connector.postgres_session import PostgresSession
from datawarehouse_connector.utils import logger

# Factory Pattern Implementation
class DatabaseSessionFactory:
    def __init__(self, data):
        self.data = data

    def get_session(self):
        db_source = self.data.get(DB_SOURCE)
        if db_source == SNOWFLAKE:
            return SnowflakeSession(self.data).create_session()
        elif db_source == REDSHIFT:
            return RedshiftSession(self.data).create_session()
        elif db_source == POSTGRESQL:
            return PostgresSession(self.data).create_session()
        else:
            logger.error(RED + " DataSource " + self.data.get(DB_SOURCE) + " not supported at the moment... " + ENDC)
            return None



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from datawarehouse_connector.constants import *
from datawarehouse_connector.utils import logger

class PostgresSession:
    def __init__(self, data):
        self.data = data

    def create_session(self):
        try:
            username = self.data.get(USERNAME)
            host = self.data.get(HOST)
            password = self.data.get(PASSWORD)
            database = self.data.get(DATABASE)
            db_connection_string = (
                f"postgresql+{PSYCOPG2_KEY}://{username}:{password}@{host}:5432/{database}"
            )
            engine = create_engine(db_connection_string)
            Session = sessionmaker(bind=engine)
            session = Session()
            if session:
                result = session.execute("SELECT version()").fetchone()
                logger.info(f"PostgreSQL version: {result[0]}")
                return session
        except Exception as e:
            logger.exception(RED + " Failed to create PostgreSQL session " + ENDC)
            return None
